@@ -23,7 +23,7 @@ def minimize(fun,
              bounds=None, 
              x0=None, 
              max_evaluations = 100000, 
-             stop_fitness = -math.inf, 
+             stop_fitness = -np.inf, 
              M = 1,
              popsize = 0,
              stall_criterion = 0, 
