@@ -26,7 +26,7 @@ def minimize(fun,
              popsize = 32, 
              max_evaluations = 100000, 
              workers = None,
-             stop_fitness = -math.inf, 
+             stop_fitness = -np.inf, 
              rg = Generator(MT19937()),
              runid=0,
              normalize = False,
@@ -93,8 +93,8 @@ def minimize(fun,
         input_sigma=input_sigma()
     if np.ndim(input_sigma) > 0:
         input_sigma = np.mean(input_sigma)   
-    parfun = None if (workers is None or workers <= 1) else parallel(fun, workers)
     array_type = ct.c_double * dim   
+    parfun = None if (workers is None or workers <= 1) else parallel(fun, workers)  
     c_callback_par = call_back_par(callback_par(fun, parfun))
     res = np.empty(dim+4)
     res_p = res.ctypes.data_as(ct.POINTER(ct.c_double))
@@ -157,13 +157,6 @@ class CRFMNES_C:
             Initial step size.
         popsize = int, optional
             CMA-ES population size.
-        max_evaluations : int, optional
-            Forced termination after ``max_evaluations`` function evaluations.
-        workers : int or None, optional
-            If workers > 1, function evaluation is performed in parallel for the whole population. 
-            Useful for costly objective functions but is deactivated for parallel retry.  
-        stop_fitness : float, optional 
-             Limit for fitness value. If reached minimize terminates.
         rg = numpy.random.Generator, optional
             Random generator for creating random guesses.
         runid : int, optional
@@ -259,4 +252,3 @@ tellCRFMNES_C.restype = ct.c_int
 
 populationCRFMNES_C = libcmalib.populationCRFMNES_C
 populationCRFMNES_C.argtypes = [ct.c_void_p, ct.POINTER(ct.c_double)]
-
