@@ -25,7 +25,7 @@ def minimize(fun,
              bounds = None, 
              popsize = None, 
              max_evaluations = 100000, 
-             stop_fitness = -math.inf, 
+             stop_fitness = -np.inf, 
              pbest = 0.7,
              f0 = 0.0,
              cr0 = 0.0,
